use crate::{schema, UIDCError};
use microrm::{prelude::*, schema::entity::EntityID};

#[derive(Debug)]
pub struct GroupInterface;

#[derive(Debug, clap::Subcommand)]
pub enum GroupCommands {
    Create { name: String },
}

impl microrm::cli::EntityInterface for GroupInterface {
    type Error = UIDCError;
    type Entity = schema::Group;
    type Context = microrm::schema::Stored<schema::Realm>;
    type CustomCommand = GroupCommands;

    fn run_custom(
        ctx: &Self::Context,
        cmd: Self::CustomCommand,
        query_ctx: impl Queryable<EntityOutput = Self::Entity> + Insertable<Self::Entity>,
    ) -> Result<(), Self::Error> {
        match cmd {
            GroupCommands::Create { name } => {
                query_ctx.insert(schema::Group {
                    realm: ctx.id(),
                    shortname: name,
                    users: Default::default(),
                    roles: Default::default(),
                })?;
            }
        }

        Ok(())
    }

    fn should_override(
        _entity: &'static str,
        field: &'static str,
        _role: microrm::cli::ValueRole,
    ) -> bool {
        if field == "realm" {
            true
        } else {
            false
        }
    }

    fn override_for(
        ctx: &Self::Context,
        _entity: &'static str,
        field: &'static str,
        _role: microrm::cli::ValueRole,
    ) -> String {
        if field == "realm" {
            format!("{}", ctx.id().into_raw())
        } else {
            unreachable!()
        }
    }
}
